EnemyBuffs_Options_Default = {
	enabled = true;
	targetPatch = true;
	useDefaultBuffs = true;
	useTempIconForUnknownBuffs = true;
	tempIconForUnknownBuffs = "Interface\\Icons\\INV_Misc_QuestionMark";
};

EnemyBuffs_Options = {};

EnemyBuffs_Buffs = {
};

EnemyBuffs_BuffTextures = {
};

function EnemyBuffs_IsEnemyNPCUnit(unit)
	if ( not UnitExists(unit) ) then
		return false;
	end
	if ( not UnitPlayerControlled(unit) ) then
		local reaction = UnitReaction(unit, "player");
		if ( reaction ) then
			if (reaction > 2) then
				return false;
			else
				return true;
			end
		end
		return false;
	else
		return false;
	end
end

function EnemyBuffs_UnitBuff(unit, index)
	if ( not unit ) or ( unit ~= "target" ) then
		return EnemyBuffs_Saved_UnitBuff(unit, index);
	end
	if ( not EnemyBuffs_IsEnemyNPCUnit(unit) ) then
		return EnemyBuffs_Saved_UnitBuff(unit, index);
	end
	if ( EnemyBuffs_Options.enabled ) then
		return EnemyBuffs_BuffTextures[index];
	end
	return nil;
end

function EnemyBuffs_GameTooltip_SetUnitBuff(tooltip, unit, index)
	if ( not EnemyBuffs_Options.enabled ) or ( not EnemyBuffs_Options.targetPatch ) then
		return EnemyBuffs_Saved_GameTooltip_SetUnitBuff(tooltip, unit, index);
	end
	if ( unit == "target" ) and ( EnemyBuffs_IsEnemyNPCUnit(unit) ) then
		local buff = EnemyBuffs_Buffs[index];
		if ( not buff ) then
			tooltip:Hide();
		else
			local obj = getglobal(tooltip:GetName().."TextLeft1");
			obj:SetText(buff);
			obj:Show();
			local bt = EnemyBuffs_BuffType[buff];
			obj = getglobal(tooltip:GetName().."TextRight1");
			if ( bt ) then
				obj:SetText(bt);
				obj:Show();
			else
				obj:Hide();
			end
			tooltip:Show();
		end
	else
		EnemyBuffs_Saved_GameTooltip_SetUnitBuff(tooltip, unit, index);
	end
end

function EnemyBuffs_ResetBuffs()
	EnemyBuffs_Buffs = {};
	EnemyBuffs_BuffTextures = {};
	TargetDebuffButton_Update();
end

function EnemyBuffs_OnLoad()
	local f = EnemyBuffsFrame;
	f:RegisterEvent("VARIABLES_LOADED");
	f:RegisterEvent("PLAYER_TARGET_CHANGED");
	for k, v in EnemyBuffs_ChatMessages do
		f:RegisterEvent(k);
	end
	EnemySpellDetector_AddListener(EnemySpellDetector_Type_Gain, "EnemyBuffs_AddBuff");
	
	EnemyBuffs_Saved_UnitBuff = UnitBuff;
	UnitBuff = EnemyBuffs_UnitBuff;
	
	EnemyBuffs_Saved_GameTooltip_SetUnitBuff = GameTooltip.SetUnitBuff;
	GameTooltip.SetUnitBuff = EnemyBuffs_GameTooltip_SetUnitBuff;
	
	EnemyBuffs_PrepareForGFind();
end

function EnemyBuffs_OnEvent()
	if ( event == "PLAYER_TARGET_CHANGED" ) then
		EnemyBuffs_TargetChanged();
		return;
	end
	if ( event == "VARIABLES_LOADED" ) then
		EnemyBuffs_HandleVariablesLoaded();
		return;
	end
	if ( EnemyBuffs_ChatMessages[event] ) then
		if ( EnemyBuffs_Options.enabled ) then
			EnemyBuffs_HandleEvent(event, arg1);
		end
		return;
	end
end

function EnemyBuffs_OnUpdate()
end

function EnemyBuffs_HandleVariablesLoaded()
	for k, v in EnemyBuffs_Options_Default do
		if ( EnemyBuffs_Options[k] == nil ) then
			EnemyBuffs_Options[k] = v;
		end
	end
	
	if ( EnemyBuffs_UserBuffs ) and ( type(EnemyBuffs_UserBuffs) == "table" ) then
		for k, v in EnemyBuffs_UserBuffs do
			EnemyBuffs_BuffMap[k] = v;
		end
	end
end

function EnemyBuffs_PrepareForGFind()
	for event, list in EnemyBuffs_ChatMessages do
		for key, value in list do
			if ( not value.pattern ) then
				value.pattern = EnemySpellDetector_GlobalStringTogfind(getglobal(value.msg));
				if ( EnemySpellDetector_OptimizeMemory ) then
					value.msg = nil;
				end
			end
		end
	end
end

function EnemyBuffs_HandleEvent(event, msg)
	local value = EnemyBuffs_ChatMessages[event];
	if ( table.getn(value) <= 0 ) then
		return;
	end
	local performer = nil;
	local action = nil;
	local target = nil;

	for key, template in value do
		for a1, a2, a3, a4, a5, a6, a7, a8, a9 in string.gfind(msg, template.pattern) do
			function getIndex(i) if ( not i ) then return nil; elseif (i == 1) then return a1; elseif (i == 2) then return a2; elseif (i == 3) then return a3; elseif (i == 4) then return a4; elseif (i == 5) then return a5; elseif (i == 6) then return a6; elseif (i == 7) then return a7; elseif (i == 8) then return a8; elseif (i == 9) then return a9; else return nil; end end
			performer = getIndex(template.performerIndex);
			action = getIndex(template.actionIndex);
			target = getIndex(template.targetIndex);
			return EnemyBuffs_RemoveBuff(template.type, performer, action, target);
		end
	end
end

function EnemyBuffs_RemoveBuff(type, performer, action, target)
	local name = UnitName("target");
	if ( not name ) then
		return false;
	end
	local otherName = target;
	if ( not otherName ) then
		otherName = performer;
	end
	if ( not otherName ) then
		return false;
	end
	if ( name ~= otherName ) then
		return false;
	end
	local foundPos = nil;
	for k, v in EnemyBuffs_Buffs do
		if ( v == action ) then
			foundPos = k;
			break;
		end
	end
	if ( foundPos ) then
		table.remove(EnemyBuffs_Buffs, foundPos);
		table.remove(EnemyBuffs_BuffTextures, foundPos);
		EnemyBuffs_BuffsRemoved(action);
	end
	return true;
end

function EnemyBuffs_AddBuff(performer, action, target)
	if ( not EnemyBuffs_Options.enabled ) then
		return;
	end
	local name = UnitName("target");
	if ( not name ) then
		return false;
	end
	local otherName = target;
	if ( not otherName ) then
		otherName = performer;
	end
	if ( not otherName ) then
		return false;
	end
	if ( name ~= otherName ) then
		return false;
	end
	local texture = EnemyBuffs_BuffMap[action];
	if ( not texture ) and ( EnemyBuffs_Options.useTempIconForUnknownBuffs ) then
		texture = EnemyBuffs_Options.tempIconForUnknownBuffs;
	end
	if ( not texture ) then
		return;
	end
	if ( type(texture) == "table" ) then
		texture = texture.t;
		if ( not texture ) then
			return;
		end
	end
	for k, v in EnemyBuffs_Buffs do
		if ( v == action ) then
			EnemyBuffs_BuffsAdded(action);
			return;
		end
	end
	table.insert(EnemyBuffs_Buffs, action);
	table.insert(EnemyBuffs_BuffTextures, texture);
	EnemyBuffs_BuffsAdded(action);
end

-- feel free to hook me! :)
function EnemyBuffs_BuffsAdded(buffName)
	if ( EnemyBuffs_Options.targetPatch ) then
		TargetDebuffButton_Update();
	end
end

-- feel free to hook me! :)
function EnemyBuffs_TargetChanged()
	EnemyBuffs_ResetBuffs();
	EnemyBuffs_SetupDefaultBuffsForTarget();
	if ( EnemyBuffs_Options.targetPatch ) then
		TargetDebuffButton_Update();
	end
end

-- feel free to hook me! :)
function EnemyBuffs_BuffsRemoved(buffName)
	if ( EnemyBuffs_Options.targetPatch ) then
		TargetDebuffButton_Update();
	end
end


function EnemyBuffs_SetupDefaultBuffsForTarget()
	if ( not EnemyBuffs_Options.useDefaultBuffs ) then
		return;
	end
	if ( not EnemyBuffs_IsEnemyNPCUnit("target") ) then
		return;
	end
	if ( UnitIsDeadOrGhost("target") ) then
		return;
	end
	local name = UnitName("target");
	if ( name ) then
		local defaultList = EnemyBuffs_DefaultBuffs[name];
		if ( defaultList ) then
			for k, v in defaultList do
				EnemyBuffs_AddBuff(name, v);
			end
		end
	end
end